@@ -35,12 +35,18 @@
                 expr;                \
         } while G_UNLIKELY (errno == EINTR);
 
+#define GDM_SYSTEMD_SESSION_REQUIRE_ONLINE 0
+
 GQuark gdm_common_error_quark (void);
 #define GDM_COMMON_ERROR gdm_common_error_quark()
 
 typedef char * (*GdmExpandVarFunc) (const char *var,
                                     gpointer user_data);
 
+typedef void   (*GdmLoadEnvVarFunc) (const char *var,
+                                     const char *value,
+                                     gpointer user_data);
+
 G_BEGIN_DECLS
 
 int            gdm_wait_on_pid           (int pid);
@@ -48,6 +54,12 @@ int            gdm_wait_on_and_disown_pid (int pid,
                                            int timeout);
 int            gdm_signal_pid            (int pid,
                                           int signal);
+
+gboolean       gdm_find_display_session (GPid        pid,
+                                         const uid_t uid,
+                                         char      **out_session_id,
+                                         GError    **error);
+
 gboolean       gdm_get_pwent_for_name    (const char     *name,
                                           struct passwd **pwentp);
 
@@ -79,6 +91,10 @@ gboolean      gdm_activate_session_by_id (GDBusConnection *connection,
                                           const char      *seat_id,
                                           const char      *session_id);
 
+void          gdm_load_env_d              (GdmLoadEnvVarFunc load_env_func,
+                                           GdmExpandVarFunc  expand_func,
+                                           gpointer          user_data);
+
 G_END_DECLS
 
 #endif /* _GDM_COMMON_H */
